import java.util.ArrayList;


public class TestEfficiency {
	static final int repeatInsert = 2;
	static final int repeatFrequency = 200000;
	static final int repeatDiffWords = 100000;
	static final int repeatMeanWordCount = 100000;
	static final int repeatWordsFreq = 5000;
	static final int repeatWordsStart = 5000;

  static final int prime = 101;
	
	public static void main(String[] args) {
		Out.println("--- 1. SortedWordList ---");
		testEfficiency(new SortedWordList());
		Out.println("--- 2. TreeWordList ---");
		testEfficiency(new TreeWordList());
	}
  static void testEfficiency(WordList list) {
    String fn;        // file name for efficiency test
    ArrayList words;  // words in file
    int size;         // amount of words in ArrayList
    String word;      // temorary word (file, ArrayList)
    WordList[] wlarr; // array of wordlists for testing purpose
    WordList wl;      // single wordlist for testing purpose
    String wordStart; // start of a word in the list
    int i, j;         // loop variables
    int freq;         // frequency of words
    long start1;      // start time
    long start2;      // intermediate time
    long end;         // end time
    // intermediate - start => WordList independant time
    // end - intermediate => total time
    // (end - intermediate) - (intermediate - start) => total time, Wordlist dependant

		fn = "Alice.txt";
    // Open file and read content into ArrayList
    In.open(fn);
    if (!In.done()) {
      return;
    }
    words = new ArrayList();
    char ch = In.peek();
    while (ch != In.eof) {
      if (Character.isLetter(ch)) {
        word = In.readIdentifier();
        words.add(word);
      }
      else {
        In.read();
      }
      ch = In.peek();
    }
    In.close();

    size = words.size();
    Out.println("File " + fn + " read (" + size + " words).");

    // Test inserting words
    wlarr = new WordList[repeatInsert];
    for (i = 0; i < repeatInsert; i ++) {
      wlarr[i] = list;
    }
    start1 = System.currentTimeMillis();
    for (i = 0; i < repeatInsert; i ++) {
      for (j = 0; j < size; j ++) {
        word = (String)words.get(j);
      }
    }
    start2 = System.currentTimeMillis();
    for (i = 0; i < repeatInsert; i ++) {
      for (j = 0; j < size; j ++) {
        word = (String)words.get(j);
        wlarr[i].insert(word);
      }
    }
    end = System.currentTimeMillis();
    wl = wlarr[0];
    Out.println("Inserting " + size + " words needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatInsert +
            " milliseconds (dw = " + wl.nOfDifferentWords() + ", mean = " +
            wl.meanWordCount() + " | " + (end - start2) + "/" + (start2 - start1) + ")");

    // Test getFrequency
    start1 = System.currentTimeMillis();
    for (i = 0; i < repeatFrequency; i ++) {
      word = (String)words.get((i * prime) % size);
    }
    start2 = System.currentTimeMillis();
    for (i = 0; i < repeatFrequency; i ++) {
      word = (String)words.get((i * prime) % size);
      wl.getFrequency(word);
    }
    end = System.currentTimeMillis();
    Out.println("getFrequency needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatFrequency +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test nOfDifferentWords
    start1 = System.currentTimeMillis();
    for (i = 0; i < repeatDiffWords; i ++) {
    }
    start2 = System.currentTimeMillis();
    for (i = 0; i < repeatDiffWords; i ++) {
      wl.nOfDifferentWords();
    }
    end = System.currentTimeMillis();
    Out.println("nOfDifferentWords needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatDiffWords +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test meanWordCount
    start1 = System.currentTimeMillis();
    for (i = 0; i < repeatMeanWordCount; i ++) {
    }
    start2 = System.currentTimeMillis();
    for (i = 0; i < repeatMeanWordCount; i ++) {
      wl.meanWordCount();
    }
    end = System.currentTimeMillis();
    Out.println("meanWordCount needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatMeanWordCount +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test getWords (1)
    start1 = System.currentTimeMillis();
    for (i = 0; i < repeatWordsFreq; i ++) {
      freq = i % 399 + 1;
    }
    start2 = System.currentTimeMillis();
    for (i = 0; i < repeatWordsFreq; i ++) {
      freq = i % 399 + 1;
      wl.getWords(freq);
    }
    end = System.currentTimeMillis();
    Out.println("getWords (1-400) needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatWordsFreq +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test getWords (2)
    start1 = System.currentTimeMillis();
    for (i = 0; i < repeatWordsFreq; i ++) {
      freq = i % 19 + 1;
    }
    start2 = System.currentTimeMillis();
    for (i = 0; i < repeatWordsFreq; i ++) {
      freq = i % 19 + 1;
      wl.getWords(freq);
    }
    end = System.currentTimeMillis();
    Out.println("getWords (1-20) needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatWordsFreq +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");

    // Test getWordsStartingWith
    start1 = System.currentTimeMillis();
    for (i = 0; i < repeatWordsStart; i ++) {
      wordStart = (String)words.get((i * prime) % size);
      wordStart = wordStart.substring(0, Math.min(i % 5 + 1, wordStart.length()));
    }
    start2 = System.currentTimeMillis();
    for (i = 0; i < repeatWordsStart; i ++) {
      wordStart = (String)words.get((i * prime) % size);
      wordStart = wordStart.substring(0, Math.min(i % 5 + 1, wordStart.length()));
      wl.getWordsStartingWith(wordStart);
    }
    end = System.currentTimeMillis();
    Out.println("getWordsStartingWith needs " +
            ((double)(end - start2 - (start2 - start1))) / repeatWordsStart +
            " milliseconds (" +(end - start2) + "/" + (start2 - start1) + ")");
  }
	
}
